-- This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.
-- To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/
-- or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.

-- This library is intended to provide a wrapper around async frame creation
-- Certain parts of the MHF UI are long to build and the watchdog sulks
-- This library is to help "stream" the UI pieces in after the important part is on screen

-- The ideas are very similar to LibAsyncTextures

local LibAsyncFrames, privateVars = ...

Library = Library or {}
Library.LibAsyncFrames = LibAsyncFrames

local lfr = Library.LibFrameRecycler

local queuedRequests = {}
local currentQueuePosition = 0

local processingEvents = false;
local function EnableProcessing()
    if (not processingEvents) then
        Command.Event.Attach(Event.System.Update.Begin, LibAsyncFrames.ProcessQueue, "LibAsyncFrames", -50);
        processingEvents = true;
    end
end

local function DisableProcessing()
    if (processingEvents) then
        Command.Event.Detach(Event.System.Update.Begin, LibAsyncFrames.ProcessQueue, "LibAsyncFrames", -50);
        processingEvents = false;
    end
    currentQueuePosition = 0
    queuedRequests = {}
end


function LibAsyncFrames.AyncLoadFrames(frameRequests, callback, parent, AddonId)
    assert(type(frameRequests) == "table")
    assert(type(callback) == "function")
    assert(parent)
    assert(AddonId)

    local queueItem = { framesRequests=frameRequests, callback = callback, parent = parent, createdFrames=nil, AddonId = AddonId }

    table.insert(queuedRequests, queueItem)

    EnableProcessing()
end

-- this function is all a co-routine, the event drives it when spare CPU is available,
-- it creates a frame at a time and yields

local localthread = nil

local function ProcessQueue()
    while (currentQueuePosition < #queuedRequests) do
        currentQueuePosition = currentQueuePosition + 1

        local queuedItem =  queuedRequests[currentQueuePosition]

        -- we have to convert from an iterator into a non-iterator table
        local walkableTable = {}

        for key, value in pairs(queuedItem.framesRequests) do
            table.insert(walkableTable, {key, value})
        end

        local FrameResults = {}
        local currentFrameType = 0
        while (currentFrameType < #walkableTable) do
            currentFrameType = currentFrameType + 1
            local frameType = walkableTable[currentFrameType]

            local returnedFrames = {}
            local frameCount = frameType[2]
            while (frameCount > 0) do
                local frame = lfr.CreateFrame(frameType[1], "AsyncFrame", queuedItem.parent)
                table.insert(returnedFrames, frame)
                coroutine.yield(true)
                frameCount = frameCount - 1
            end
            FrameResults[frameType[1]] = returnedFrames
        end

        FrameResults.CreateFrame = LibAsyncFrames.CreateFrame
        Utility.Dispatch(function () return queuedItem.callback(FrameResults) end,
                queuedItem.AddonId,
                "Callback from LibAsyncFrames")
        coroutine.yield(true)
    end

    localthread = nil

    coroutine.yield(false)
end


local InspectSystemWatchdog = Inspect.System.Watchdog
local InspectTimeReal = Inspect.Time.Real

local MinTimeNeeded = 0.020

function LibAsyncFrames.ProcessQueue()
    if (localthread == nil) then
        localthread = coroutine.create(ProcessQueue)
    end

    if(InspectSystemWatchdog() < MinTimeNeeded) then
        return
    end

    local threadStatus, result
    repeat
        threadStatus, result = coroutine.resume(localthread)
    until (threadStatus == false or result == false or InspectSystemWatchdog() < MinTimeNeeded)

    if (threadStatus == false) then
        print("Fatal error creating frames: ", result)
        DisableProcessing()
    end

    if (result == false) then
        DisableProcessing()
    end
end


local timeStart
local function testCallBack(results)
    local timeEnd = InspectTimeReal()

    print ("Time Taken:", timeEnd - timeStart)

    for _, values in pairs(results) do
        if (type(values) == "table") then
            for _, value in ipairs(values) do
                lfr.RecycleFrame(value)
            end
        end
    end
end

local context
function LibAsyncFrames.TestLibrary()
    if (context == nil) then
        context = UI.CreateContext("TestLibContext")
    end
    local frames = {
        Text = 10,
        Frame = 10,
    }
    timeStart = InspectTimeReal()
    LibAsyncFrames.AyncLoadFrames(frames, testCallBack, context, "LibAsyncFrames")
    LibAsyncFrames.AyncLoadFrames(frames, testCallBack, context, "LibAsyncFrames")
    LibAsyncFrames.AyncLoadFrames(frames, testCallBack, context, "LibAsyncFrames")
    LibAsyncFrames.AyncLoadFrames(frames, testCallBack, context, "LibAsyncFrames")
    LibAsyncFrames.AyncLoadFrames(frames, testCallBack, context, "LibAsyncFrames")
end

-- wrapper func to take a frame from the generated ones, and return it
-- if needed it will also create a frame, but shouldn't need to, and will warn if it does
function LibAsyncFrames.CreateFrame(frames, type, name, parent)
    local frameArray = frames[type]
    if (#frameArray > 0) then

        local frame = table.remove(frameArray)
        frame:SetParent(parent)
        return frame
    end

    print("LibAsyncFrames: Not enough frames created of type ", type)

    return lfr.CreateFrame(type, name, parent)
end