-- Setup library data structures
local lib = {
  debug = false,
  recycled = {},
  counter = 0
}

-- Helper Functions
local function FindContext(frame)
  local parent = frame:GetParent()
  if parent == nil then
    return frame
  else
    return FindContext(parent)
  end
end

-- Library Functions
function lib.CreateFrame(frameType, namePrefix, parent)
  local context = FindContext(parent)
  if lib.recycled[context] == nil then
    lib.recycled[context] = {}
  end
  local recycled = lib.recycled[context]

  if recycled[frameType] == nil then
    recycled[frameType] = {}
  end
  local frames = recycled[frameType]

  local frame = table.remove(frames)
  if frame == nil then
    lib.counter = lib.counter + 1
    frame = UI.CreateFrame(frameType, namePrefix..tostring(lib.counter), parent)
    frame.__lfr_frameType = frameType
    if lib.debug then
      print("Created new "..frameType.." frame: "..frame:GetName())
    end
  else
    frame:SetParent(parent)
    if lib.debug then
      print("Used recycled frame: "..frame:GetName())
    end
  end

  frame:SetVisible(true)

  return frame
end

function lib.RecycleFrame(frame)
  local frameType = frame.__lfr_frameType
  -- don't try to recycle a frame which was not created by this library
  if frameType then
    local context = FindContext(frame)
    if lib.recycled[context] == nil then
      lib.recycled[context] = {}
    end
    local recycled = lib.recycled[context]

    if recycled[frameType] == nil then
      recycled[frameType] = {}
    end

    table.insert(recycled[frameType], frame)
    frame:SetVisible(false)
    frame:SetParent(context)
    frame:SetBackgroundColor(0, 0, 0, 0)
    frame:ClearAll()
    frame:ClearWidth()
    frame:ClearHeight()
    frame:SetLayer(0)
    frame:SetStrata("main")
    frame:SetMouseMasking("full")
    if frame:GetSecureMode() == "restricted" then
      frame:SetMouseoverUnit(nil)
    end
    frame:SetSecureMode("normal")
    for k, v in pairs(frame.Event) do
      if v ~= nil then
        if lib.debug then
          print(frame:GetName()..": Clearing event handler: "..k)
        end
        frame.Event[k] = nil
      end
    end
    for k, v in pairs(frame) do
      if k ~= "Event" and not k:match("__lfr_.+") then
        frame[k] = nil
      end
    end
    if frameType == "Text" then
      frame:SetFontColor(1, 1, 1, 1)
      frame:SetFontSize(12)
      frame:SetText("")
      frame:SetWordwrap(false)
    end
    if frameType == "RiftButton" then
      frame:SetEnabled(true)
      frame:SetSkin("default")
      frame:SetText("")
    end
    if frameType == "RiftCheckbox" then
      frame:SetChecked(false)
      frame:SetEnabled(true)
    end
    if frameType == "RiftScrollbar" then
      frame:SetEnabled(true)
      frame:SetOrientation("vertical")
      frame:SetRange(1, 100)
      frame:SetPosition(1)
      frame:SetThickness(64)
    end
    if frameType == "RiftSlider" then
      frame:SetEnabled(true)
      frame:SetRange(1, 100)
      frame:SetPosition(1)
    end
    if frameType == "RiftTextfield" then
      frame:SetText("")
      frame:SetSelection()
    end
    if frameType == "RiftWindow" then
      frame:SetController("border")
    end
    if lib.debug then
      print("Recycled frame: " .. frame:GetName())
    end
  end

  -- try to recycle the frame's children even if it wasn't recyclable
  local children = frame:GetChildren()
  for k, v in pairs(children) do
    lib.RecycleFrame(k)
  end
end

-- Test Functions
local function assertSameFrame(f1, f2)
  if f1 ~= f2 then
    error("Assertion failed: Frame1 '"..f1:GetName().."' ("..tostring(f1)..") is not the same as Frame2 '"..f2:GetName().."' ("..tostring(f2)..")")
  end
end

local function assertNotSameFrame(f1, f2)
  if f1 == f2 then
    error("Assertion failed: Both frames given are Frame '"..f1:GetName().."' ("..tostring(f1)..")")
  end
end

function lib.TestCreateFrame()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  local f2 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  assertNotSameFrame(f1, f2)
end

function lib.TestRecycleFrame()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  Library.LibFrameRecycler.RecycleFrame(f1)
  local f2 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  assertSameFrame(f1, f2)
end

function lib.TestRecycleAcrossContexts()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  Library.LibFrameRecycler.RecycleFrame(f1)
  local c2 = UI.CreateContext("context2")
  local f2 = Library.LibFrameRecycler.CreateFrame("Frame", "context2frame", c2)
  assertNotSameFrame(f1, f2)
end

function lib.TestRecycleChildren()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  local f2 = Library.LibFrameRecycler.CreateFrame("Frame", "contest1framechild", f1)
  assertNotSameFrame(f1, f2)
  Library.LibFrameRecycler.RecycleFrame(f1)
  assertSameFrame(c1, f1:GetParent())
  assertSameFrame(c1, f2:GetParent())
end

function lib.TestRecycleDifferentTypes()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Text", "context1frame", c1)
  Library.LibFrameRecycler.RecycleFrame(f1)
  local f2 = Library.LibFrameRecycler.CreateFrame("RiftTextfield", "context1frame", c1)
  assertNotSameFrame(f1, f2)
end

function lib.TestRecycleAllTypes()
  local types = { "Frame", "Mask", "Text", "Texture", "RiftButton", "RiftCheckbox", "RiftScrollbar", "RiftSlider",
                  "RiftTextfield", "RiftWindow" }
  local c1 = UI.CreateContext("context1")
  for i, type in ipairs(types) do
    local f1 = Library.LibFrameRecycler.CreateFrame(type, "context1frame", c1)
    Library.LibFrameRecycler.RecycleFrame(f1)
  end
end

function lib.TestEventHandlers()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Text", "context1frame", c1)
  f1.Event.Size = function() end
  Library.LibFrameRecycler.RecycleFrame(f1)
  local f2 = Library.LibFrameRecycler.CreateFrame("Text", "context1frame", c1)
  assertSameFrame(f1, f2)
  if f1.Event.Size ~= nil then
    error("Assertion failed: Event handler was not cleared for "..f1:GetName())
  end
end

function lib.TestRemoveExtraMembers()
  local c1 = UI.CreateContext("context1")
  local f1 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  f1.extra = 123
  Library.LibFrameRecycler.RecycleFrame(f1)
  local f2 = Library.LibFrameRecycler.CreateFrame("Frame", "context1frame", c1)
  assertSameFrame(f1, f2)
  if f1.Event == nil or f1.__lfr_frameType == nil then
    error("Assertion failed: Required members were removed")
  end
  if f1.extra ~= nil then
    error("Assertion failed: Extra members were not removed")
  end
end

function lib.Test()
  lib.debug = true
  for k, v in pairs(lib) do
    if k:match("Test.+") then
      print("=== "..k.." ===")
      v()
    end
  end
  print("All tests passed")
  lib.debug = false
end

-- Make the library globally accessible
Library = Library or {}
Library.LibFrameRecycler = lib
